<?php
namespace Academy\Classes;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Tools {

	private function get_memory_limit() {
		// WP memory limit.
		$wp_memory_limit = WP_MEMORY_LIMIT;
		if ( function_exists( 'memory_get_usage' ) ) {
			// phpcs:ignore WordPress.PHP.NoSilencedErrors.Discouraged
			$wp_memory_limit = max( $wp_memory_limit, @ini_get( 'memory_limit' ) );
		}
		return $wp_memory_limit;
	}

	private function get_curl_version() {
		$curl_version = '';
		if ( function_exists( 'curl_version' ) ) {
			$curl_version = curl_version();
			$curl_version = $curl_version['version'] . ', ' . $curl_version['ssl_version'];
		}
		return $curl_version;
	}

	public function get_wordpress_environment_status() {
		return array(
			'home_url'                  => get_option( 'home' ),
			'site_url'                  => get_option( 'siteurl' ),
			'wp_version'                => get_bloginfo( 'version' ),
			'academy_version'            => ACADEMY_VERSION,
			'wp_multisite'              => is_multisite(),
			'wp_memory_limit'           => $this->get_memory_limit(),
			'wp_debug_mode'             => ( defined( 'WP_DEBUG' ) && WP_DEBUG ),
			'wp_cron'                   => ! ( defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON ),
			'language'                  => get_locale(),
			'external_object_cache'     => wp_using_ext_object_cache(),
		);
	}

	public function get_message( $type, $value ) {
		if ( 'wordpress_memory_limit' === $type ) {
			if ( $value < 67108864 ) {
				return sprintf( esc_html__( '%1$s - We recommend setting memory to at least 64MB. See: %2$s', 'academy' ), esc_html( size_format( $environment['wp_memory_limit'] ) ), '<a href="https://codex.wordpress.org/Editing_wp-config.php#Increasing_memory_allocated_to_PHP" target="_blank">' . esc_html__( 'Increasing memory allocated to PHP', 'academy' ) . '</a>' );
			}
			return $value;
		} elseif ( 'php_version' === $type ) {
			if ( version_compare( $value, '5.6', '<' ) && version_compare( $value, '7.2', '>=' ) ) {
				return __( 'We recommend using PHP version 7.2 or above for greater performance and security.', 'academy' );
			}
			return $value;
		} elseif ( 'mysql_version' === $type ) {
			if ( version_compare( $value['number'], '5.6', '<' ) && ! strstr( $value['string'], 'MariaDB' ) ) {
				return sprintf( esc_html__( '%1$s - We recommend a minimum MySQL version of 5.6. See: %2$s', 'academy' ), esc_html( $value['string'] ), '<a href="https://wordpress.org/about/requirements/" target="_blank">' . esc_html__( 'WordPress requirements', 'academy' ) . '</a>' );
			}
			return $value;
		} elseif ( 'default_timezone' === $type ) {
			if ( 'UTC' !== $value ) {
				return sprintf( esc_html__( 'Default timezone is %s - it should be UTC', 'academy' ), esc_html( $value ) );
			}
			return $value;
		} elseif ( 'fsockopen_or_curl_enabled' === $type ) {
			if ( ! $value ) {
				return esc_html__( 'Your server does not have fsockopen or cURL enabled - PayPal IPN and other scripts which communicate with other servers will not work. Contact your hosting provider.', 'academy' );
			}
			return $value;
		} elseif ( 'domdocument_enabled' === $type ) {
			if ( ! $value ) {
				return sprintf( esc_html__( 'Your server does not have the %s class enabled - HTML/Multipart emails, and also some extensions, will not work without DOMDocument.', 'academy' ), '<a href="https://php.net/manual/en/class.domdocument.php">DOMDocument</a>' );

			}
			return $value;
		} elseif ( 'gzip_enabled' === $type ) {
			if ( ! $value ) {
				return sprintf( esc_html__( 'Your server does not support the %s function - this is required to use the GeoIP database from MaxMind.', 'academy' ), '<a href="https://php.net/manual/en/zlib.installation.php">gzopen</a>' );

			}
			return;
		} elseif ( 'multibyte_string' === $type ) {
			if ( ! $value ) {
				return sprintf( esc_html__( 'Your server does not support the %s functions - this is required for better character encoding. Some fallbacks will be used instead for it.', 'academy' ), '<a href="https://php.net/manual/en/mbstring.installation.php">mbstring</a>' );
			}
			return $value;
		}//end if
		return '';
	}

	public function get_server_environment_status() {
		// phpcs:disable
		global $wpdb;
		return array(
			'server_info'               => isset( $_SERVER['SERVER_SOFTWARE'] ) ? wp_unslash( $_SERVER['SERVER_SOFTWARE'] ) : '',
			'php_version'               => phpversion(),
			'php_post_max_size'         => @ini_get( 'post_max_size' ),
			'php_max_execution_time'    => @ini_get( 'max_execution_time' ),
			'php_max_input_vars'        => @ini_get( 'max_input_vars' ),
			'curl_version'              => $this->get_curl_version(),
			'suhosin_installed'         => extension_loaded( 'suhosin' ),
			'max_upload_size'           => wp_max_upload_size(),
			'mysql_version'             => $wpdb->db_version(),
			'default_timezone'          => date_default_timezone_get(),
			'fsockopen_or_curl_enabled' => ( function_exists( 'fsockopen' ) || function_exists( 'curl_init' ) ),
			'soapclient_enabled'        => class_exists( 'SoapClient' ),
			'domdocument_enabled'       => class_exists( 'DOMDocument' ),
			'gzip_enabled'              => is_callable( 'gzopen' ),
			'mbstring_enabled'          => extension_loaded( 'mbstring' ),
		);
		// phpcs:enable
	}
}
