<?php
namespace Academy;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Academy\Admin\Settings;

class Migration {

	public static function init() {
		$self = new self();
		$self->run_migration();
	}

	public function run_migration() {
		$academy_version = get_option( 'academy_version' );
		// Migration for addons multi instructor. when all users come to v1.3 then it will be deleted
		if ( version_compare( $academy_version, '1.2.15', '<=' ) ) {
			$addons = [
				'multi_instructor' => true,
			];
			add_option( ACADEMY_ADDONS_SETTINGS_NAME, wp_json_encode( $addons ) );
		}

		// Fix Lesson Status Issue - when all users come to v1.3.5 then it will be deleted
		$this->migrate_1_3_5( $academy_version );

		// Fix Course Wishlist Issue - when all users come to v1.4.0 then it will be deleted
		$this->migrate_1_4_0( $academy_version );

		// Fix Course Announcement & QA Migration - when all users come to v1.6.0 then it will be deleted
		$this->migrate_1_6_0( $academy_version );

		// Save Version Number, flash role management and save permalink
		if ( ACADEMY_VERSION !== $academy_version ) {
			Settings::save_settings();
			update_option( 'academy_version', ACADEMY_VERSION );
			update_option( 'academy_flash_role_management', true );
			update_option( 'academy_required_rewrite_flush', Helper::get_time() );
		}
		// Flash Role
		if ( get_option( 'academy_flash_role_management' ) ) {
			$Installer = new \Academy\Installer();
			$Installer->add_role();
			delete_option( 'academy_flash_role_management' );
		}
		// current user have administrator role and not have instructor role then assign instructor role
		$user = new \WP_User( get_current_user_id() );
		if ( in_array( 'administrator', $user->roles, true ) && ! in_array( 'academy_instructor', $user->roles, true ) ) {
			$user->add_role( 'academy_instructor' );
		}
	}

	public function migrate_1_3_5( $academy_version ) {
		if ( version_compare( $academy_version, '1.2.15', '>=' ) && version_compare( $academy_version, '1.3.5', '<' ) ) {
			global $wpdb;
			$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->prefix}academy_lessons SET lesson_status=%s WHERE lesson_status= %s", 'publish', 'draft' ) );
		}
	}

	public function migrate_1_4_0( $academy_version ) {
		$user_id = get_current_user_id();
		if ( ! get_user_meta( $user_id, 'academy_is_user_migrate_completed_topics', true ) ) {
			global $wpdb;
			// if current user have no enrolled course then return it.
			$enrolled_course_ids = \Academy\Helper::get_enrolled_courses_ids_by_user( $user_id );
			if ( ! count( $enrolled_course_ids ) ) {
				update_user_meta( $user_id, 'academy_is_user_migrate_completed_topics', true );
				return;
			}

			$topic_lists = $wpdb->get_results(
				$wpdb->prepare(
					"SELECT meta_key, meta_value FROM $wpdb->usermeta WHERE meta_key LIKE %s AND user_id = %d",
					'academy_completed_topic_id_%',
					$user_id
				)
			);

			$quiz = [];
			$lesson = [];
			foreach ( $topic_lists as $topic_item ) {
				$topic_id = (int) str_replace( 'academy_completed_topic_id_', '', $topic_item->meta_key );
				if ( 'academy_quiz' === get_post_type( $topic_id ) ) {
					$quiz[ $topic_id ] = $topic_item->meta_value;
				} else {
					$lesson[ $topic_id ] = $topic_item->meta_value;
				}
			}

			// if current user have no completed lesson or quiz then return it
			if ( ! count( $quiz ) && ! count( $lesson ) ) {
				update_user_meta( $user_id, 'academy_is_user_migrate_completed_topics', true );
				return;
			}

			// Saved user meta to get enrolled course id then update user meta.
			foreach ( $enrolled_course_ids as $enrolled_course_id ) {
				$curriculums = wp_list_pluck( get_post_meta( $enrolled_course_id, 'academy_course_curriculum', true ), 'topics' );
				$curriculums = call_user_func_array( 'array_merge', $curriculums );
				$option_name = 'academy_course_' . $enrolled_course_id . '_completed_topics';
				$saved_topics_lists = (array) json_decode( get_user_meta( $user_id, $option_name, true ), true );
				foreach ( $curriculums as $curriculum ) {
					if ( isset( $lesson[ $curriculum['id'] ] ) && 'lesson' === $curriculum['type'] ) {
						if ( ! isset( $saved_topics_lists['lesson'][ $curriculum['id'] ] ) ) {
							$saved_topics_lists['lesson'][ $curriculum['id'] ] = $lesson[ $curriculum['id'] ];
						}
					} elseif ( isset( $quiz[ $curriculum['id'] ] ) && 'quiz' === $curriculum['type'] ) {
						if ( ! isset( $saved_topics_lists['quiz'][ $curriculum['id'] ] ) ) {
							$saved_topics_lists['quiz'][ $curriculum['id'] ] = $quiz[ $curriculum['id'] ];
						}
					}
				}
				update_user_meta( $user_id, $option_name, wp_json_encode( $saved_topics_lists ) );
			}
			update_user_meta( $user_id, 'academy_is_user_migrate_completed_topics', true );
		}//end if
	}

	public function migrate_1_6_0( $academy_version ) {
		if ( version_compare( $academy_version, '1.6.0', '<' ) ) {
			global $wpdb;
			$courses = $wpdb->get_results(
				$wpdb->prepare("SELECT ID 
				FROM {$wpdb->posts} 
				WHERE post_type = %s 
				AND post_status = %s", 'academy_courses', 'publish')
			);
			if ( is_array( $courses ) ) {
				foreach ( $courses as $course ) {
					update_post_meta( $course->ID, 'academy_is_enabled_course_qa', true );
					update_post_meta( $course->ID, 'academy_is_enabled_course_announcements', true );
				}
			}
		}
	}
}
